-- Controls an elevator on a prismatic joint with two buttons
-- The elevator continues at given speed until reaching the joint limit or the timer runs out
pTimer = GetFloatParam("timer", 0.1)
pSpeed = GetFloatParam("speed", 4)

function init()
	motor = FindJoint("motor")
	motor2 = FindJoint("motor2")
	motor3 = FindJoint("motor3")
	limitMin, limitMax = GetJointLimits(motor)
	base = FindShape("base")
	base_body = GetShapeBody(base)
	SetBodyDynamic(base_body, false)
	down = FindShape("down")
	down2 = FindShape("down2")
	up = FindShape("up")
	up2 = FindShape("up2")
	stop = FindShape("stop")
	stop2 = FindShape("stop2")
	clickUp = LoadSound("clickup.ogg")
	clickDown = LoadSound("clickdown.ogg")

	SetShapeEmissiveScale(down, 0)
	SetShapeEmissiveScale(up, 0)

	SetTag(up, "interact", "Up")
	SetTag(up2, "interact", "Up")
	SetTag(down, "interact", "Down")
	SetTag(down2, "interact", "Down")
	
	downPressed = false
	downTimer = 0
	upPressed = false
	upTimer = 0
	stopPressed = false
	stopTimer = 0
end


function press(shape)
	PlaySound(clickDown)
	local t = GetShapeLocalTransform(shape)
	t.pos[3] = t.pos[3] - 0.05
	SetShapeLocalTransform(shape, t)
	SetShapeEmissiveScale(shape, 1)
	SetBodyDynamic(base_body, true)
	SetTag(stop, "interact", "Stop")
	SetTag(stop2, "interact", "Stop")
end


function unpress(shape)
	PlaySound(clickUp)
	local t = GetShapeLocalTransform(shape)
	t.pos[3] = t.pos[3] + 0.05
	SetShapeLocalTransform(shape, t)
	SetShapeEmissiveScale(shape, 0)
end

function tick(dt)
	downTimer = downTimer - dt
	upTimer = upTimer - dt

	if InputPressed("interact") and not downPressed and not upPressed then
		if GetPlayerInteractShape() == down then
			upTimer = 0
			downTimer = pTimer
			downPressed = true
			stopPressed = false
			press(down)
			SetJointMotor(motor2, 20.0)
			SetJointMotor(motor3, 20.0)
		end
		if GetPlayerInteractShape() == down2 then
			upTimer = 0
			downTimer = pTimer
			downPressed = true
			stopPressed = false
			press(down)
			SetJointMotor(motor2, 20.0)
			SetJointMotor(motor3, 20.0)
		end
		if GetPlayerInteractShape() == stop then
			upPressed = false
			downPressed = false
			SetJointMotor(motor, 0.0)
			SetJointMotor(motor2, -20.0)
			SetJointMotor(motor3, -20.0)			
		end
		if GetPlayerInteractShape() == stop2 then
			upPressed = false
			downPressed = false
			SetJointMotor(motor, 0.0)
			SetJointMotor(motor2, -20.0)
			SetJointMotor(motor3, -20.0)			
		end
		if GetPlayerInteractShape() == up then
			downTimer = 0
			upTimer = pTimer
			upPressed = true
			stopPressed = false
			press(up)
			SetJointMotor(motor2, 20.0)
			SetJointMotor(motor3, 20.0)
		end
		if GetPlayerInteractShape() == up2 then
			downTimer = 0
			upTimer = pTimer
			upPressed = true
			stopPressed = false
			press(up)
			SetJointMotor(motor2, 20.0)
			SetJointMotor(motor3, 20.0)
		end
	end
	
	if downTimer <= 0 then
		downTimer = 0
		if downPressed then
			unpress(down)
		end
		downPressed = false
	end
	
	
	if upTimer <= 0 then
		upTimer = 0
		if upPressed then
			unpress(up)
		end
		upPressed = false
	end

	local eps = 0.01
	if downPressed then
		SetJointMotor(motor, pSpeed)
		PlayLoop(motorSound)
		if GetJointMovement(motor) < limitMin+eps then
			downTimer = 0.0			
		end
	elseif upPressed then
		SetJointMotor(motor, -pSpeed)
		PlayLoop(motorSound)
		if GetJointMovement(motor) > limitMax-eps then
			upTimer = 0.0			
		end
	end

	if IsJointBroken(motor) then
		pTimer = GetFloatParam("timer", 0)
		pSpeed = GetFloatParam("speed", 0)
		RemoveTag(up, "interact", "Up")
		RemoveTag(up2, "interact", "Up")
		RemoveTag(down, "interact", "Down")
		RemoveTag(down2, "interact", "Down")
		RemoveTag(stop, "interact", "Stop")
		RemoveTag(stop2, "interact", "Stop")
    end
end

